@@ -161,10 +161,11 @@ static void connection_read_for_eos(server *srv, connection *con) {
 	 * it will make the client not see all our output.
 	 */
 	ssize_t len;
-	char buf[4096];
-
+	const int type = con->dst_addr.plain.sa_family;
+	char buf[16384];
 	do {
-		len = read(con->fd, buf, sizeof(buf));
+		len = fdevent_socket_read_discard(con->fd, buf, sizeof(buf),
+						  type, SOCK_STREAM);
 	} while (len > 0 || (len < 0 && errno == EINTR));
 
 	if (len < 0 && errno == EAGAIN) return;
@@ -325,6 +326,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			if (0 == http_chunk_append_file(srv, con, con->physical.path)) {
 				con->file_finished = 1;
 				if (HANDLER_ERROR != stat_cache_get_entry(srv, con, con->physical.path, &sce)) {
+					stat_cache_content_type_get(srv, con, con->physical.path, sce);
 					response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(sce->content_type));
 				}
 			}
@@ -424,7 +426,10 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		 */
 
 		if (!(con->parsed_response & (HTTP_CONTENT_LENGTH|HTTP_TRANSFER_ENCODING|HTTP_UPGRADE))) {
-			if (con->request.http_version == HTTP_VERSION_1_1) {
+			if (con->request.http_method == HTTP_METHOD_CONNECT
+			    && con->http_status == 200) {
+				/*(no transfer-encoding if successful CONNECT)*/
+			} else if (con->request.http_version == HTTP_VERSION_1_1) {
 				off_t qlen = chunkqueue_length(con->write_queue);
 				con->response.transfer_encoding = HTTP_TRANSFER_ENCODING_CHUNKED;
 				if (qlen) {
@@ -695,6 +700,7 @@ int connection_reset(server *srv, connection *con) {
 	/* config_cond_cache_reset(srv, con); */
 
 	con->header_len = 0;
+	con->async_callback = 0;
 	con->error_handler_saved_status = 0;
 	/*con->error_handler_saved_method = HTTP_METHOD_UNSET;*/
 	/*(error_handler_saved_method value is not valid unless error_handler_saved_status is set)*/
@@ -908,9 +914,26 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 		if (con->state == CON_STATE_CLOSE) {
 			con->close_timeout_ts = srv->cur_ts - (HTTP_LINGER_TIMEOUT+1);
 		} else if (revents & FDEVENT_HUP) {
-			if (fdevent_is_tcp_half_closed(con->fd)) {
+			connection_set_state(srv, con, CON_STATE_ERROR);
+		} else if (revents & FDEVENT_RDHUP) {
+			if (sock_addr_get_family(&con->dst_addr) == AF_UNIX) {
+				/* future: will getpeername() on AF_UNIX properly check if still connected? */
+				fdevent_event_clr(srv->ev, &con->fde_ndx, con->fd, FDEVENT_RDHUP);
+				con->keep_alive = 0;
+			} else if (fdevent_is_tcp_half_closed(con->fd)) {
+				/* Success of fdevent_is_tcp_half_closed() after FDEVENT_RDHUP indicates TCP FIN received,
+				 * but does not distinguish between client shutdown(fd, SHUT_WR) and client close(fd).
+				 * Remove FDEVENT_RDHUP so that we do not spin on the ready event.
+				 * However, a later TCP RST will not be detected until next write to socket.
+				 * future: might getpeername() to check for TCP RST on half-closed sockets
+				 * (without FDEVENT_RDHUP interest) when checking for write timeouts
+				 * once a second in server.c, though getpeername() on Windows might not indicate this */
+				fdevent_event_clr(srv->ev, &con->fde_ndx, con->fd, FDEVENT_RDHUP);
 				con->keep_alive = 0;
 			} else {
+				/* Failure of fdevent_is_tcp_half_closed() indicates TCP RST
+				 * (or unable to tell (unsupported OS), though should not
+				 * be setting FDEVENT_RDHUP in that case) */
 				connection_set_state(srv, con, CON_STATE_ERROR);
 			}
 		} else if (revents & FDEVENT_ERR) { /* error, connection reset */
@@ -1370,8 +1393,7 @@ int connection_state_machine(server *srv, connection *con) {
 	r = 0;
 	switch(con->state) {
 	case CON_STATE_READ:
-	case CON_STATE_CLOSE:
-		r = FDEVENT_IN;
+		r = FDEVENT_IN | FDEVENT_RDHUP;
 		break;
 	case CON_STATE_WRITE:
 		/* request write-fdevent only if we really need it
@@ -1386,9 +1408,12 @@ int connection_state_machine(server *srv, connection *con) {
 		/* fall through */
 	case CON_STATE_READ_POST:
 		if (con->conf.stream_request_body & FDEVENT_STREAM_REQUEST_POLLIN) {
-			r |= FDEVENT_IN;
+			r |= FDEVENT_IN | FDEVENT_RDHUP;
 		}
 		break;
+	case CON_STATE_CLOSE:
+		r = FDEVENT_IN;
+		break;
 	default:
 		break;
 	}
@@ -1402,6 +1427,9 @@ int connection_state_machine(server *srv, connection *con) {
 			con->is_writable = 0;
 			r |= FDEVENT_OUT;
 		}
+		if (events & FDEVENT_RDHUP) {
+			r |= FDEVENT_RDHUP;
+		}
 		if (r != events) {
 			/* update timestamps when enabling interest in events */
 			if ((r & FDEVENT_IN) && !(events & FDEVENT_IN)) {
