@@ -391,7 +391,7 @@ void fdevent_event_add(fdevents *ev, int *fde_ndx, int fd, int event) {
 	if (-1 == fd) return;
 
 	events = ev->fdarray[fd]->events;
-	if ((events & event) || 0 == event) return; /*(no change; nothing to do)*/
+	if ((events & event) == event) return; /*(no change; nothing to do)*/
 
 	events |= event;
 	if (ev->event_set) *fde_ndx = ev->event_set(ev, *fde_ndx, fd, events);
@@ -847,6 +847,8 @@ static int fdevent_open_logger_pipe(const char *logger) {
     }
     fdevent_setfd_cloexec(fds[0]);
     fdevent_setfd_cloexec(fds[1]);
+    /*(nonblocking write() from lighttpd)*/
+    if (0 != fdevent_fcntl_set_nb(NULL, fds[1])) { /*(ignore)*/ }
 
     pid = fdevent_open_logger_pipe_spawn(logger, fds[0]);
 
@@ -889,6 +891,28 @@ int fdevent_cycle_logger(const char *logger, int *curfd) {
 }
 
 
+#ifndef MSG_DONTWAIT
+#define MSG_DONTWAIT 0
+#endif
+#ifndef MSG_NOSIGNAL
+#define MSG_NOSIGNAL 0
+#endif
+
+
+ssize_t fdevent_socket_read_discard (int fd, char *buf, size_t sz, int family, int so_type) {
+  #if defined(MSG_TRUNC) && defined(__linux__)
+    if ((family == AF_INET || family == AF_INET6) && so_type == SOCK_STREAM) {
+        ssize_t len = recv(fd, buf, sz, MSG_TRUNC|MSG_DONTWAIT|MSG_NOSIGNAL);
+        if (len >= 0 || errno != EINVAL) return len;
+    }
+  #else
+    UNUSED(family);
+    UNUSED(so_type);
+  #endif
+    return read(fd, buf, sz);
+}
+
+
 #include <sys/ioctl.h>
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>  /* FIONREAD (for illumos (OpenIndiana)) */
