@@ -19,6 +19,7 @@ typedef handler_t (*fdevent_handler)(struct server *srv, void *ctx, int revents)
 #define FDEVENT_ERR    BV(3)
 #define FDEVENT_HUP    BV(4)
 #define FDEVENT_NVAL   BV(5)
+#define FDEVENT_RDHUP  BV(13)
 
 #define FDEVENT_STREAM_REQUEST          BV(0)
 #define FDEVENT_STREAM_REQUEST_BUFMIN   BV(1)
@@ -77,6 +78,8 @@ void fdevent_close_logger_pipes(void);
 void fdevent_breakagelog_logger_pipe(int fd);
 void fdevent_clr_logger_pipe_pids(void);
 
+ssize_t fdevent_socket_read_discard (int fd, char *buf, size_t sz, int family, int so_type);
+
 int fdevent_ioctl_fionread (int fd, int fdfmt, int *toread);
 
 int fdevent_connect_status(int fd);
