@@ -6,9 +6,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>
 #include <dirent.h>
-#include <stdio.h>
 
 #include <lauxlib.h>
 
