@@ -7,8 +7,6 @@
 
 #include "plugin.h"
 
-#include <assert.h>
-#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -55,6 +53,7 @@ FREE_FUNC(mod_simple_vhost_free) {
 		size_t i;
 		for (i = 0; i < srv->config_context->used; i++) {
 			plugin_config *s = p->config_storage[i];
+			if (NULL == s) continue;
 
 			buffer_free(s->document_root);
 			buffer_free(s->default_host);
