@@ -128,6 +128,18 @@ SETDEFAULTS_FUNC(mod_userdir_set_defaults) {
 		if (0 != config_insert_values_global(srv, config->value, cv, i == 0 ? T_CONFIG_SCOPE_SERVER : T_CONFIG_SCOPE_CONNECTION)) {
 			return HANDLER_ERROR;
 		}
+
+		if (!array_is_vlist(s->exclude_user)) {
+			log_error_write(srv, __FILE__, __LINE__, "s",
+					"unexpected value for userdir.exclude-user; expected list of \"suffix\"");
+			return HANDLER_ERROR;
+		}
+
+		if (!array_is_vlist(s->include_user)) {
+			log_error_write(srv, __FILE__, __LINE__, "s",
+					"unexpected value for userdir.include-user; expected list of \"suffix\"");
+			return HANDLER_ERROR;
+		}
 	}
 
 	return HANDLER_GO_ON;
