@@ -636,6 +636,7 @@ int http_request_parse(server *srv, connection *con) {
 
 					buffer_copy_string_len(con->request.uri, nuri, proto - nuri - 1);
 				} else if (!http_header_strict
+					   || (HTTP_METHOD_CONNECT == con->request.http_method && (uri[0] == ':' || light_isdigit(uri[0])))
 					   || (HTTP_METHOD_OPTIONS == con->request.http_method && uri[0] == '*' && uri[1] == '\0')) {
 					/* everything looks good so far */
 					buffer_copy_string_len(con->request.uri, uri, proto - uri - 1);
@@ -1322,14 +1323,3 @@ int http_request_parse(server *srv, connection *con) {
 
 	return 0;
 }
-
-int http_request_header_finished(server *srv, connection *con) {
-	UNUSED(srv);
-
-	if (buffer_string_length(con->request.request) < 4) return 0;
-
-	if (0 == memcmp(con->request.request->ptr + buffer_string_length(con->request.request) - 4, CONST_STR_LEN("\r\n\r\n"))) return 1;
-	if (NULL != strstr(con->request.request->ptr, "\r\n\r\n")) return 1;
-
-	return 0;
-}
